% Copyright 2016. Massachusetts Institute of Technology. Rights Reserved.
% M.I.T. hereby makes following copyrightable material available to the
% public under GNU General Public License, version 2 (GPL-2.0). A copy of
% this license is available at https://opensource.org/licenses/GPL-2.0
function [] = Brownian_Dynamics(nnodes, dt, dur)
%Brownian_Dynamics (Version 1.0)   Calculate a Brownian Dynamics trajectory
% Syntax: [] = Brownian_Dynamics(nnodes, dt, dur)
% 
% Input parameters:
%   1. nnodes, the number of nodes in a DNA nanostructure model
%   2. dt, the time step size in picoseconds
%   3. dur, the duration of the simulation in picoseconds
%
% Input files, which must exist in the currect directory:
%   1. k.mat, the 6Nx6N K matrix. Units of length, mass, and energy are
%      Angstrom (1e-10 meter), 1e-22 kilogram, and 1e-10 second,
%      respectively.
%   2. zz.mat, the 3Nx3N friction matrix that was obtained using a bead 
%      model. For each node, here we only consider three translational 
%      degrees of freedom. Units of length, mass, and energy are Angstrom,
%      Dalton, and picosecond.
%   3. z.mat, the 6Nx6N friction matrix (z) which is created using a bead 
%      model. Units of length, mass, and energy are Angstrom, Dalton, and 
%      picosecond.
%   4. nodes.txt, the node positions in Angstroms.
%
% Output files, which will be saved in the current directory:
%   1. u_saved.mat, the displacement of each node and the rigid-body
%      translation of the whole finite element model
%   2. AA_saved.mat, the rigid-body rotation of the whole finite element
%      model

% nodessur, here for DNA nanostructures, is the same as nnodes. 
nodessur=nnodes;

% Number of Brownian dynamics trajectories/simulations. By default, NoSim
% has been set to 10. 
NoSim=1;

% zz is the 3Nx3N friction matrix that was obtained using a bead model. For each node, here we only consider three translational degrees of freedom. 
load zz 

% here we calculate an A matrix which is equal to 2*KB*T*z. Please note that KB is
% the Boltzmann constant and T is the temperature. Then we decompose the A
% matrix using Cholesky decomposition to calculate a B matrix, which is used 
% later to calculate the random forces in the Brownian dynamics simulation.
A=2*0.831261932624000*293.15*zz;
BB=chol(A,'lower');

% here we create a 6Nx6N B matrix which is similar to the 3Nx3N BB matrix.
% In the B matrix, the arrays corresponding to rotational degrees of
% freedom are zero and other arrays are the same as those of the BB matrix.  
B=zeros(6*nnodes,6*nnodes);
for i=1:nnodes
    for j=1:nnodes
        B(6*(i-1)+1,6*(j-1)+1)=BB(3*(i-1)+1,3*(j-1)+1);
        B(6*(i-1)+2,6*(j-1)+1)=BB(3*(i-1)+2,3*(j-1)+1);
        B(6*(i-1)+3,6*(j-1)+1)=BB(3*(i-1)+3,3*(j-1)+1);

        B(6*(i-1)+1,6*(j-1)+2)=BB(3*(i-1)+1,3*(j-1)+2);
        B(6*(i-1)+2,6*(j-1)+2)=BB(3*(i-1)+2,3*(j-1)+2);
        B(6*(i-1)+3,6*(j-1)+2)=BB(3*(i-1)+3,3*(j-1)+2);

        B(6*(i-1)+1,6*(j-1)+3)=BB(3*(i-1)+1,3*(j-1)+3);
        B(6*(i-1)+2,6*(j-1)+3)=BB(3*(i-1)+2,3*(j-1)+3);
        B(6*(i-1)+3,6*(j-1)+3)=BB(3*(i-1)+3,3*(j-1)+3);
    end
end


% here we load the 6Nx6N K matrix and then change the units from the CanDo
% units to a unit system of Dalton for mass, picosecond for time and
% Angstrom for length. 
load k
k=k*10/1.6605;

% here we load the 6Nx6N friction matrix (z) which is created using a bead model.
load z

% we need to calculate kh, khinv, khb, and khz for our time integration
% method (trapezoidal rule ). You could read about the trapezoidal rule in
% Prof. KJ Bathe's book in 1996 (Finite Element Procedures).
kh=2/dt*z+k;
khinv=kh^-1;
khb=khinv*B;
khz=khinv*z;
kk=k;
zs=z;

% xx, yy, and zz are the coordinates of the nodes in the initial
%configuration.
xx=[];
yy=[];
zz=[];

% Reading node positions
fid=fopen('nodes.txt');
C=textscan(fid,'%*s%*d%f%f%f%*[^\n]',nnodes,'Headerlines',11);
fclose(fid)
xx=C{1};
yy=C{2};
zz=C{3};


save -v7.3 matrices B kk zs khb khz nnodes nodessur dt dur xx yy zz NoSim

clear all
close all

warning off MATLAB:fprintf:InputForPercentSIsNotOfClassChar

load matrices

% Here we have put the center of the structure at the origin at t=0. 
mean_xx=mean(xx);
mean_yy=mean(yy);
mean_zz=mean(zz);

for i=1:size(xx,1)
    xx(i)=xx(i)-mean_xx;
    yy(i)=yy(i)-mean_yy;
    zz(i)=zz(i)-mean_zz;
end

% here we calculate the number of time steps (b) in our simulation. 
b=round(dur/dt);

% In u_saved, I save the internal x-displacement, y-displacement, z-displacement, x-rotation, y-rotation, and z-rotation 
% of the nodes of the DNA nanostructure(taking out the rigid body motions). 
u_saved=[];

% In u_saved_f, I save the x-displacement, y-displacement, z-displacement, x-rotation, y-rotation, and z-rotation  
% of the nodes of the DNA nanostructure(including the rigid body motions).
u_saved_f=[];

% u_saved1 and u_saved_f1 are two intermediate matrices, which I use for
% saving my data temporarily. 
u_saved1=[];

u_saved_f1=[];

% The rigid-body rotation matrix of the current configuration of the DNA nanostructure relative to the initial configuration is saved in the AA_saved 
AA_saved=[];


% xxx, yyy, and zzz are the x-position, y-position, and z-position of the nodes of the
% DNA nanostructure (taking out the rigid body motions). 
xxx=[];
yyy=[];
zzz=[];

% xxx, yyy, and zzz are the x-position, y-position, and z-position of the nodes of the
% DNA nanostructure (including the rigid body motions).
xxxf=[];
yyyf=[];
zzzf=[];

% here I run the simulation for NoSim times. 
for ttt=1:NoSim 

    xxx1=xx;
    yyy1=yy;
    zzz1=zz;
    
    xxxf1=xx;
    yyyf1=yy;
    zzzf1=zz;
        
    % ud is a vector of the x-translational velocity, y-translational velocity, 
    % z-translational velocity, x-rotational velocity, y-rotational velocity, 
    % z-rotational velocity of the nodes of the DNA nanostructure at each time step.
    ud=zeros(size(khz,1),1);

    % u is a vector of three translational and three rotational
    % displacements of the nodes of the DNA nanostructure at each time step.
    u=zeros(size(khz,1),1);

    % We save the potential energy of the system at each time step in
    % energy. 
    energy=[];
    % We save the error of the time integration scheme at each time step in
    % eps. See Prof. KJ Bathe's book in 1996 (Finite Element Procedures)... 
    eps=[];

    % energy1 is an intermediate matrix.
    energy1=[];

    jj=1;

    % ll, lll and llll are counters.

    ll=0;

    lll=0;

    llll=0;

    % AA is the rotation matrix. It gives you the rigid body orientation of the DNA nanostructure at each time step. 
    % Please read more on the Kabsch algorithm to understand the next lines
    % well (http://en.wikipedia.org/wiki/Kabsch_algorithm). Since we do
    % linear analysis, we need to always rotate our structure at each time
    % to its original shape; but we save the orientation of the
    % structure in AA as a rotation matrix.
    
    AA=eye(3,3);
    
    % AA_saved1 is an intermediate matrices, which I use for
    % saving my data temporarily. 
    AA_saved1=[];

    % u_store and u_store1 are just intermediate matrices to save data
    % temporarily.
    u_store=zeros(size(khz,1),1);

    u_store1=zeros(size(khz,1),1);

    % QQ and PP are, respectively, Q and P matrices in
    % http://en.wikipedia.org/wiki/Kabsch_algorithm. 
    % please read there for more information. 
    QQ=[xx yy zz];

    QQ_x=mean(xx);

    QQ_y=mean(yy);

    QQ_z=mean(zz);



for i=1:size(xx,1)

    QQ(i,1)=QQ(i,1)-QQ_x;

    QQ(i,2)=QQ(i,2)-QQ_y;

    QQ(i,3)=QQ(i,3)-QQ_z;

end



PP=QQ';

% Here we solve the Brownian dynamics equations for b time steps. 
for i=1:b

    llll=llll+1;
    lll=lll+1;
    ll=ll+1;

    % r is a vector of random numbers. We could multiply them by the B matrix 
    % to calculate the random forces in the Brownian dynamics simulation.  
    r=(1/dt)^0.5*randn(6*nodessur,1);

    % Please read more on the trapezoidal rule to understand the next
    % lines.
    uz=2/dt*u+ud;

    u11=khb*r+khz*uz;

    ud11=(u11-u)*2/dt-ud;

    u=u11;

    ud=ud11;
   
    kuu=kk*u;

    fm=1/2*u'*kuu;

    energy1=[energy1 fm];
    
     % every 10 iterations I run the following "if" block. You could
     % change that. This block is for rotating back the DNA
     % nanostructure to its original shape. Please read more on the Kabsch
     % algorithm.
     if lll==10

        u_x=0;
        u_y=0;
        u_z=0;

        u_x=mean(u(1:6:size(khz,1),1));
        u_y=mean(u(2:6:size(khz,1),1));
        u_z=mean(u(3:6:size(khz,1),1));

        for k=1:size(khz,1)/6
            u_store1(6*(k-1)+1,1)=u_x;
            u_store1(6*(k-1)+2,1)=u_y;
            u_store1(6*(k-1)+3,1)=u_z;
        end

        u=u-u_store1;

        for k=1:size(PP,2)
            PP(1,k)=QQ(k,1)+u(6*(k-1)+1,1);
            PP(2,k)=QQ(k,2)+u(6*(k-1)+2,1);
            PP(3,k)=QQ(k,3)+u(6*(k-1)+3,1);
        end

        cov=PP*QQ;

        [Vc,Sc,Wc]=svd(cov);

        II=eye(3);

        if (det(cov) < 0)
            II(3,3) = -1 ;
        end

        A=Wc*II*Vc';
        
        % You could print out the AA matrix here. The AA matrix is the
        % rigid body rotation matrix of the whole DNA nanostructure at each time step. It gives you the orientation of the DNA nanostructure.  
        AA=AA*A^-1;

        for k=1:size(khz,1)/6

            vvvv=[QQ(k,1)+u(6*(k-1)+1,1);QQ(k,2)+u(6*(k-1)+2,1);QQ(k,3)+u(6*(k-1)+3,1)]-A*[QQ(k,1)+u(6*(k-1)+1,1);QQ(k,2)+u(6*(k-1)+2,1);QQ(k,3)+u(6*(k-1)+3,1)];

            u(6*(k-1)+1,1)=u(6*(k-1)+1,1)-vvvv(1);
            u(6*(k-1)+2,1)=u(6*(k-1)+2,1)-vvvv(2);
            u(6*(k-1)+3,1)=u(6*(k-1)+3,1)-vvvv(3);

            vvvv=A*[ud(6*(k-1)+1,1);ud(6*(k-1)+2,1);ud(6*(k-1)+3,1)];

            ud(6*(k-1)+1,1)=vvvv(1);
            ud(6*(k-1)+2,1)=vvvv(2);
            ud(6*(k-1)+3,1)=vvvv(3);
        end
        u_store=u_store+u_store1;
        u_store1=zeros(size(khz,1),1);
        lll=0;

     end
    
     % The next "if" block is for saving the positions of the nodes of
     % the DNA nanostructure. Every 10 iterations, I save the data. 
    if ll==10
        
        AA_saved1=[AA_saved1 AA];
        
        eps=[eps norm(B*r-zs*ud11-kuu)/norm(B*r)];
        jj=jj+1;
        
        xxx1=[xxx1 xx+u(1:6:size(u,1))];
        yyy1=[yyy1 yy+u(2:6:size(u,1))];
        zzz1=[zzz1 zz+u(3:6:size(u,1))];
         
        u_saved1=[u_saved1 u];
        
        uu=u;
    
        for k=1:size(khz,1)/6
            vvvv=AA*[QQ(k,1)+u(6*(k-1)+1,1);QQ(k,2)+u(6*(k-1)+2,1);QQ(k,3)+u(6*(k-1)+3,1)]-[QQ(k,1);QQ(k,2);QQ(k,3)];
            u(6*(k-1)+1,1)=vvvv(1);
            u(6*(k-1)+2,1)=vvvv(2);
            u(6*(k-1)+3,1)=vvvv(3);
        end
    
        u=u+u_store;
        ttt
        jj
        
        xxxf1=[xxxf1 xx+u(1:6:size(u,1))];
        yyyf1=[yyyf1 yy+u(2:6:size(u,1))];
        zzzf1=[zzzf1 zz+u(3:6:size(u,1))];
        
        
        u_saved_f1=[u_saved_f1 u];
        u=uu;
        ll=0;
   
    end
    
    % The next "if" block is for saving the data every round(b/10)
    % iterations. You do not need to touch it. 
    if llll==round(b/10)
       AA_saved=[AA_saved AA_saved1];
       AA_saved1=[];
       energy=[energy energy1];
       energy1=[];
       llll=0;

       u_saved=[u_saved u_saved1];
       u_saved1=[];
       
       u_saved_f=[u_saved_f u_saved_f1];
       u_saved_f1=[];
       
       xxx=[xxx xxx1];
       xxx1=[];
       yyy=[yyy yyy1];
       yyy1=[];
       zzz=[zzz zzz1];
       zzz1=[];
       
       xxxf=[xxxf xxxf1];
       xxxf1=[];
       yyyf=[yyyf yyyf1];
       yyyf1=[];
       zzzf=[zzzf zzzf1];
       zzzf1=[];
       
       llll=0;
     end
        
    
end
       AA_saved=[AA_saved AA_saved1];
       AA_saved1=[]; 

       energy=[energy energy1];
       energy1=[];
       llll=0;

       u_saved=[u_saved u_saved1];
       u_saved1=[];
       
       u_saved_f=[u_saved_f u_saved_f1];
       u_saved_f1=[];
       
       xxx=[xxx xxx1];
       xxx1=[];
       yyy=[yyy yyy1];
       yyy1=[];
       zzz=[zzz zzz1];
       zzz1=[];
       
       xxxf=[xxxf xxxf1];
       xxxf1=[];
       yyyf=[yyyf yyyf1];
       yyyf1=[];
       zzzf=[zzzf zzzf1];
       zzzf1=[];


save -v7.3 AA_saved AA_saved        
       
% save -v7.3 energy energy eps

save -v7.3 u_saved u_saved u_saved_f

% save -v7.3 rrr xxx yyy zzz xxxf yyyf zzzf 

end

save -v7.3 AA_saved AA_saved 

% save -v7.3 energy energy eps 

save -v7.3 u_saved u_saved u_saved_f

% save -v7.3 rrr xxx yyy zzz xxxf yyyf zzzf    

end